import asyncio
from typing import Optional

from fastapi import APIRouter, HTTPException
import pam
from pydantic import BaseModel

from common_util import cmd_line_args, generic_ok_response
from domain_socket_listener import (
    MsgFromGlintServer,
    create_subscriber,
    delete_subscriber,
    get_socket_msg,
)
from glint_session import (
    non_block_init,
    start_desktop,
    start_glint,
    start_pulseaudio,
    start_xvfb,
)


auth_router = APIRouter()


class AuthData(BaseModel):
    username: str
    password: str
    width: Optional[int] = 1280
    height: Optional[int] = 1024
    depth: Optional[int] = 24
    desktop: Optional[str] = None


async def wait_for_server_ready_msg(msg_queue):
    while True:
        msg = await get_socket_msg(msg_queue)
        if msg == MsgFromGlintServer.SERVER_START_FAIL.value:
            raise HTTPException(status_code=400, detail="Failed to start Glint Server")
        elif msg == MsgFromGlintServer.SERVER_READY.value:
            print("!!! msg == MsgFromGlintServer.SERVER_READY.value:")  # noqa: T201
            return


@auth_router.post("/auth")
async def auth(auth_data: AuthData):
    # Извлечь логин пароль. Проверить их.
    authorized = pam.authenticate(auth_data.username, auth_data.password)
    if not authorized:
        raise HTTPException(status_code=401, detail="Wrong credentials")

    non_block_init(auth_data.username)

    # Запуск графической сессии
    print("Beg auth")  # noqa: T201
    loop = asyncio.get_event_loop()
    asyncio.ensure_future(
        loop.run_in_executor(None, start_xvfb, auth_data.username, auth_data.width, auth_data.height, auth_data.depth)
    )
    await asyncio.sleep(0.5)
    desktop = auth_data.desktop if auth_data.desktop else cmd_line_args.desktop
    asyncio.ensure_future(loop.run_in_executor(None, start_desktop, auth_data.username, desktop))
    await asyncio.sleep(0.5)

    domain_socket_msg_queue = create_subscriber()

    asyncio.ensure_future(loop.run_in_executor(None, start_glint, auth_data.username))
    asyncio.ensure_future(loop.run_in_executor(None, start_pulseaudio, auth_data.username))

    # Wait for message from glint server
    try:
        await asyncio.wait_for(wait_for_server_ready_msg(domain_socket_msg_queue), 12)
    except asyncio.TimeoutError:
        pass

    delete_subscriber(domain_socket_msg_queue)
    print("End auth")  # noqa: T201
    return generic_ok_response
